@@ -123,6 +123,9 @@ struct _GladeProjectPrivate
 	time_t  mtime;         /* last UTC modification time of file, or 0 if it could not be read */
 
 	GladeProjectFormat format; /* file format */
+
+	GHashTable *target_versions_major; /* target versions by catalog */
+	GHashTable *target_versions_minor; /* target versions by catalog */
 };
 
 
@@ -216,6 +219,8 @@ glade_project_finalize (GObject *object)
 	g_hash_table_destroy (project->priv->widget_names_allocator);
 	g_hash_table_destroy (project->priv->widget_old_names);
 	g_hash_table_destroy (project->priv->resources);
+	g_hash_table_destroy (project->priv->target_versions_major);
+	g_hash_table_destroy (project->priv->target_versions_minor);
 
 	G_OBJECT_CLASS (glade_project_parent_class)->finalize (object);
 }
@@ -460,11 +465,40 @@ glade_project_changed_impl (GladeProject *project,
 /*******************************************************************
                             Initializers
  *******************************************************************/
+static void
+glade_project_set_target_version (GladeProject *project,
+				  const gchar  *catalog,
+				  gint          major,
+				  gint          minor)
+{
+	g_hash_table_insert (project->priv->target_versions_major,
+			     g_strdup (catalog),
+			     GINT_TO_POINTER (major));
+	g_hash_table_insert (project->priv->target_versions_minor,
+			     g_strdup (catalog),
+			     GINT_TO_POINTER (minor));
+}
+
+static void
+glade_project_get_target_version (GladeProject *project,
+				  const gchar  *catalog,
+				  gint         *major,
+				  gint         *minor)
+{
+	*major = GPOINTER_TO_INT 
+		(g_hash_table_lookup (project->priv->target_versions_major,
+				      catalog));
+	*minor = GPOINTER_TO_INT 
+		(g_hash_table_lookup (project->priv->target_versions_minor,
+				      catalog));
+}
+
 static void
 glade_project_init (GladeProject *project)
 {
 	GladeProjectPrivate *priv;
-	
+	GList *list;
+
 	project->priv = priv = GLADE_PROJECT_GET_PRIVATE (project);	
 
 	priv->path = NULL;
@@ -476,6 +510,7 @@ glade_project_init (GladeProject *project)
 	priv->undo_stack = NULL;
 	priv->prev_redo_item = NULL;
 	priv->first_modification = NULL;
+
 	priv->widget_names_allocator = g_hash_table_new_full (g_str_hash,
 							      g_str_equal,
 							      g_free, 
@@ -491,7 +526,28 @@ glade_project_init (GladeProject *project)
 
 	priv->unsaved_number = glade_id_allocator_allocate (get_unsaved_number_allocator ());
 
-	priv->format = GLADE_PROJECT_FORMAT_LIBGLADE;
+	priv->format = GLADE_PROJECT_FORMAT_GTKBUILDER;
+
+
+	priv->target_versions_major = g_hash_table_new_full (g_str_hash,
+							     g_str_equal,
+							     g_free,
+							     NULL);
+	priv->target_versions_minor = g_hash_table_new_full (g_str_hash,
+							     g_str_equal,
+							     g_free,
+							     NULL);
+
+	for (list = glade_app_get_catalogs(); list; list = list->next)
+	{
+		GladeCatalog *catalog = list->data;
+
+		/* Set default target to catalog version */
+		glade_project_set_target_version (project,
+						  glade_catalog_get_name (catalog),
+						  glade_catalog_get_major_version (catalog),
+						  glade_catalog_get_minor_version (catalog));
+	}
 }
 
 static void
@@ -805,18 +861,21 @@ glade_project_fix_object_props (GladeProject *project)
 }
 
 static gboolean
-loadable_interface (GladeXmlNode *root_node, const gchar *path)
+glade_project_read_requires (GladeProject *project,
+			     GladeXmlNode *root_node, 
+			     const gchar  *path)
 {
 
 	GString      *string = g_string_new (NULL);
 	GladeXmlNode *node;
 	gchar        *required_lib;
 	gboolean      loadable = TRUE;
+	gint          major, minor;
 
 	for (node = glade_xml_node_get_children (root_node); 
 	     node; node = glade_xml_node_next (node))
 	{
-		/* Skip "requires" tags */
+		/* Skip non "requires" tags */
 		if (!glade_xml_node_verify_silent (node, GLADE_XML_TAG_REQUIRES))
 			continue;
 
@@ -824,6 +883,9 @@ loadable_interface (GladeXmlNode *root_node, const gchar *path)
 		     glade_xml_get_property_string_required (node, GLADE_XML_TAG_LIB, 
 							     NULL)) != NULL)
 		{
+			/* Dont mention gtk+ as a required lib in 
+			 * the generated glade file
+			 */
 			if (!glade_catalog_is_loaded (required_lib))
 			{
 				if (!loadable)
@@ -832,13 +894,44 @@ loadable_interface (GladeXmlNode *root_node, const gchar *path)
 				g_string_append (string, required_lib);
 				loadable = FALSE;
 			}
-			
+			else if (glade_xml_get_property_version (node, 
+								 GLADE_XML_TAG_VERSION, 
+								 &major, &minor))
+				glade_project_set_target_version
+					(project, required_lib, major, minor);
+
+			g_free (required_lib);
 		}
 	}
 
+
+	/* We use a different tag to save target version metadata in libglade files */
+	for (node = glade_xml_node_get_children (root_node); 
+	     node; node = glade_xml_node_next (node))
+	{
+		/* Skip non "requires" tags */
+		if (!glade_xml_node_verify_silent (node, GLADE_XML_TAG_REQUIRES_LIBGLADE_EXTRA))
+			continue;
+
+		if ((required_lib = 
+		     glade_xml_get_property_string_required (node, GLADE_XML_TAG_LIB, 
+							     NULL)) != NULL)
+		{
+			if (glade_xml_get_property_version (node, 
+							    GLADE_XML_TAG_VERSION, 
+							    &major, &minor))
+				glade_project_set_target_version
+					(project, required_lib, major, minor);
+
+
+			g_free (required_lib);
+		}
+	}
+
+
 	if (!loadable)
 		glade_util_ui_message (glade_app_get_window(),
-				       GLADE_UI_ERROR,
+				       GLADE_UI_ERROR, NULL,
 				       _("Failed to load %s.\n"
 					 "The following required catalogs are unavailable: %s"),
 				       path, string->str);
@@ -897,7 +990,7 @@ glade_project_load_from_file (GladeProject *project, const gchar *path)
 	/* XXX Need to load project->priv->comment ! */
 	glade_project_read_comment (project, doc);
 
-	if (loadable_interface (root, path) == FALSE)
+	if (glade_project_read_requires (project, root, path) == FALSE)
 	{
 		glade_xml_context_free (context);
 		return FALSE;
@@ -1665,10 +1758,13 @@ glade_project_update_comment (GladeProject *project)
 static GladeXmlContext *
 glade_project_write (GladeProject *project)
 {
+	GladeProjectFormat fmt;
 	GladeXmlContext *context;
 	GladeXmlDoc     *doc;
 	GladeXmlNode    *root, *req_node, *comment_node;
 	GList           *required, *list;
+	gint             major, minor;
+	gchar           *version;
 
 	doc     = glade_xml_doc_new ();
 	context = glade_xml_context_new (doc, NULL);
@@ -1680,16 +1776,51 @@ glade_project_write (GladeProject *project)
 
 	/* XXX Need to append this to the doc ! not the ROOT !
 	   glade_xml_node_append_child (root, comment_node); */
+	fmt = glade_project_get_format (project);
 
 	if ((required = glade_project_required_libs (project)) != NULL)
 	{
 		for (list = required; list; list = list->next)
 		{
-			req_node = glade_xml_node_new (context, GLADE_XML_TAG_REQUIRES);
-			glade_xml_node_append_child (root, req_node);
-			glade_xml_node_set_property_string (req_node, 
-							    GLADE_XML_TAG_LIB, 
-							    (gchar *)list->data);
+			glade_project_get_target_version (project, (gchar *)list->data, 
+							  &major, &minor);
+			
+			version = g_strdup_printf ("%d.%d", major, minor);
+
+			/* Write the standard requires tag */
+			if (fmt == GLADE_PROJECT_FORMAT_GTKBUILDER ||
+			    (fmt == GLADE_PROJECT_FORMAT_LIBGLADE &&
+			     strcmp ("gtk+", (gchar *)list->data)))
+			{
+				req_node = glade_xml_node_new (context, GLADE_XML_TAG_REQUIRES);
+				glade_xml_node_append_child (root, req_node);
+				glade_xml_node_set_property_string (req_node, 
+								    GLADE_XML_TAG_LIB, 
+								    (gchar *)list->data);
+			
+
+				if (fmt != GLADE_PROJECT_FORMAT_LIBGLADE)
+					glade_xml_node_set_property_string 
+						(req_node, GLADE_XML_TAG_VERSION, version);
+			}
+
+			/* Add extra metadata for libglade */
+			if (fmt == GLADE_PROJECT_FORMAT_LIBGLADE)
+			{
+				req_node = glade_xml_node_new
+					(context, GLADE_XML_TAG_REQUIRES_LIBGLADE_EXTRA);
+				glade_xml_node_append_child (root, req_node);
+
+				glade_xml_node_set_property_string (req_node, 
+								    GLADE_XML_TAG_LIB, 
+								    (gchar *)list->data);
+			
+				glade_xml_node_set_property_string (req_node, 
+								    GLADE_XML_TAG_VERSION, 
+								    version);
+			}
+			g_free (version);
+
 		}
 		g_list_foreach (required, (GFunc)g_free, NULL);
 		g_list_free (required);
@@ -1773,6 +1904,267 @@ glade_project_move_resources (GladeProject *project,
 	g_list_free (resources);
 }
 
+static void
+glade_project_target_version_for_adaptor (GladeProject        *project, 
+					  GladeWidgetAdaptor  *adaptor,
+					  gint                *major,
+					  gint                *minor)
+{
+	gchar   *catalog = NULL;
+
+	g_object_get (adaptor, "catalog", &catalog, NULL);
+	glade_project_get_target_version (project, catalog, major, minor);
+	g_free (catalog);
+}
+
+static void
+glade_project_verify_property (GladeProperty  *property, 
+			       const gchar    *path_name,
+			       GString        *string,
+			       gboolean        packing)
+{
+	GladeWidgetAdaptor *adaptor;
+	gint target_major, target_minor;
+	gchar *catalog;
+
+	adaptor = GLADE_WIDGET_ADAPTOR (property->klass->origin_handle);
+	
+	g_object_get (adaptor, "catalog", &catalog, NULL);
+	glade_project_target_version_for_adaptor (property->widget->project, adaptor, 
+						  &target_major,
+						  &target_minor);
+	
+	if (target_major < property->klass->version_since_major ||
+	    (target_major == property->klass->version_since_major &&
+	     target_minor < property->klass->version_since_minor))
+		g_string_append_printf
+				(string,
+				 packing ?
+				 _("[%s] Packing property '%s' of object class '%s' was "
+				   "introduced in %s %d.%d\n") :
+				 _("[%s] Property '%s' of object class '%s' was "
+				   "introduced in %s %d.%d\n"),
+				 path_name,
+				 property->klass->name, 
+				 adaptor->title, catalog,
+				 property->klass->version_since_major,
+				 property->klass->version_since_minor);
+
+	g_free (catalog);
+		
+}
+
+
+static void
+glade_project_verify_properties (GladeWidget  *widget, 
+				 const gchar  *path_name,
+				 GString      *string)
+{
+	GList *list;
+	GladeProperty *property;
+
+	for (list = widget->properties; list; list = list->next)
+	{
+		property = list->data;
+		glade_project_verify_property (property, path_name, string, FALSE);
+	}
+
+	for (list = widget->packing_properties; list; list = list->next)
+	{
+		property = list->data;
+
+		g_assert (widget->parent);
+		property = list->data;
+		glade_project_verify_property (property, path_name, string, FALSE);
+	}
+}
+
+static void
+glade_project_verify_signal (GladeWidget  *widget,
+			     GladeSignal  *signal,
+			     const gchar  *path_name,
+			     GString      *string)
+{
+	GladeSignalClass *signal_class;
+	gint target_major, target_minor;
+	gchar *catalog;
+
+	signal_class = 
+		glade_widget_adaptor_get_signal_class (widget->adaptor,
+						       signal->name);
+	g_assert (signal_class);
+	g_assert (signal_class->adaptor);
+	
+	g_object_get (signal_class->adaptor, "catalog", &catalog, NULL);
+	glade_project_target_version_for_adaptor (widget->project, 
+						  signal_class->adaptor, 
+						  &target_major,
+						  &target_minor);
+
+	if (target_major < signal_class->version_since_major ||
+	    (target_major == signal_class->version_since_major &&
+	     target_minor < signal_class->version_since_minor))
+		g_string_append_printf
+			(string, 
+			 _("[%s] Signal '%s' of object class '%s' was "
+			   "introduced in %s %d.%d\n"),
+			 path_name,
+			 signal->name,
+			 signal_class->adaptor->title, 
+			 catalog,
+			 signal_class->version_since_major,
+			 signal_class->version_since_minor);
+
+	g_free (catalog);
+}
+
+
+static void
+glade_project_verify_signals (GladeWidget  *widget, 
+			      const gchar  *path_name,
+			      GString      *string)
+{
+	GladeSignal      *signal;
+	GList *signals, *list;
+	
+	if ((signals = glade_widget_get_signal_list (widget)) != NULL)
+	{
+		for (list = signals; list; list = list->next)
+		{
+			signal = list->data;
+			glade_project_verify_signal (widget, signal, path_name, string);
+		}
+		g_list_free (signals);
+	}	
+}
+
+static gboolean
+glade_project_verify_dialog (GladeProject *project,
+			     GString      *string,
+			     gboolean      saving)
+{
+	GtkWidget     *swindow;
+	GtkWidget     *textview;
+	GtkWidget     *expander;
+	GtkTextBuffer *buffer;
+	gchar         *name;
+	gboolean ret;
+
+	swindow   = gtk_scrolled_window_new (NULL, NULL);
+	textview  = gtk_text_view_new ();
+	buffer    = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
+	expander  = gtk_expander_new (_("Details"));
+
+	gtk_text_buffer_set_text (buffer, string->str, -1);
+
+	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (swindow),
+					       textview);
+	gtk_container_add (GTK_CONTAINER (expander), swindow);
+	gtk_widget_show_all (expander);
+
+	gtk_widget_set_size_request (swindow, 800, -1);
+	
+	name = glade_project_get_name (project);
+	ret = glade_util_ui_message (glade_app_get_window (),
+				     saving ? GLADE_UI_YES_OR_NO : GLADE_UI_INFO,
+				     expander,
+				     saving ? 
+				     _("Project %s has errors, save anyway ?") :
+				     _("Project %s has deprecated widgets "
+				       "and/or version mismatches."), name);
+	g_free (name);
+
+	return ret;
+}
+
+static void
+glade_project_verify_adaptor_supported (GladeProject       *project,
+					GladeWidgetAdaptor *adaptor,
+					const gchar        *path_name,
+					GString            *string,
+					gboolean            saving)
+{
+	GladeWidgetAdaptor *adaptor_iter;
+	gint                target_major, target_minor;
+	gchar              *catalog = NULL;
+
+	for (adaptor_iter = adaptor; adaptor_iter;
+	     adaptor_iter = glade_widget_adaptor_get_parent_adaptor (adaptor_iter))
+	{
+
+		g_object_get (adaptor_iter, "catalog", &catalog, NULL);
+		glade_project_target_version_for_adaptor (project, adaptor_iter, 
+							  &target_major,
+							  &target_minor);
+
+		if (target_major < GWA_VERSION_SINCE_MAJOR (adaptor_iter) ||
+		    (target_major == GWA_VERSION_SINCE_MAJOR (adaptor_iter) &&
+		     target_minor < GWA_VERSION_SINCE_MINOR (adaptor_iter)))
+			g_string_append_printf
+				(string, 
+				 _("[%s] Object class '%s' was introduced in %s %d.%d\n"),
+				 path_name, adaptor_iter->title, catalog,
+				 GWA_VERSION_SINCE_MAJOR (adaptor_iter),
+				 GWA_VERSION_SINCE_MINOR (adaptor_iter));
+
+		if (project->priv->format == GLADE_PROJECT_FORMAT_GTKBUILDER &&
+		    GWA_BUILDER_UNSUPPORTED (adaptor_iter))
+			g_string_append_printf
+				(string,
+				 _("[%s] Object class '%s' of catalog '%s' is not supported "
+				   "by GtkBuilder\n"),
+				 path_name, adaptor_iter->title, catalog);
+
+		if (!saving && GWA_DEPRECATED (adaptor_iter))
+			g_string_append_printf
+				(string, 
+				 _("[%s] Object class '%s' of catalog '%s' is deprecated\n"),
+				 path_name, adaptor_iter->title, catalog);
+
+		g_free (catalog);
+	}
+
+}
+
+
+static gboolean
+glade_project_verify (GladeProject *project, 
+		      gboolean      saving)
+{
+	GString     *string = g_string_new (NULL);
+	GladeWidget *widget;
+	GList       *list;
+	gboolean     ret = TRUE;
+	gchar       *path_name;
+
+	for (list = project->priv->objects; list; list = list->next)
+	{
+		widget = glade_widget_get_from_gobject (list->data);
+
+		path_name = glade_widget_generate_path_name (widget);
+
+		glade_project_verify_adaptor_supported (project, widget->adaptor, 
+							path_name, string, saving);
+		glade_project_verify_properties (widget, path_name, string);
+		glade_project_verify_signals (widget, path_name, string);
+
+		g_free (path_name);
+	}
+
+	if (string->len > 0)
+	{
+		ret = glade_project_verify_dialog (project, string, saving);
+
+		if (!saving)
+			ret = FALSE;
+	}
+
+	g_string_free (string, TRUE);
+
+	return ret;
+}
+
+
 /**
  * glade_project_save:
  * @project: a #GladeProject
@@ -1791,6 +2183,9 @@ glade_project_save (GladeProject *project, const gchar *path, GError **error)
 	gchar           *canonical_path;
 	gint             ret;
 
+	if (!glade_project_verify (project, TRUE))
+		return FALSE;
+
 	context = glade_project_write (project);
 	doc     = glade_xml_context_get_doc (context);
 	ret     = glade_xml_doc_save (doc, path);
@@ -2355,3 +2750,253 @@ glade_project_get_format (GladeProject *project)
 
 	return project->priv->format;
 }
+
+
+
+static void
+format_libglade_button_clicked (GtkWidget *widget,
+				GladeProject *project)
+{
+	glade_project_set_format (project, GLADE_PROJECT_FORMAT_LIBGLADE);
+}
+
+static void
+format_builder_button_clicked (GtkWidget *widget,
+			       GladeProject *project)
+{
+	glade_project_set_format (project, GLADE_PROJECT_FORMAT_GTKBUILDER);
+}
+
+static void
+target_button_clicked (GtkWidget *widget,
+		       GladeProject *project)
+{
+	GladeTargetableVersion *version = 
+		g_object_get_data (G_OBJECT (widget), "version");
+	gchar *catalog = 
+		g_object_get_data (G_OBJECT (widget), "catalog");
+
+	glade_project_set_target_version (project,
+					  catalog,
+					  version->major,
+					  version->minor);
+}
+
+static void
+verify_clicked (GtkWidget    *button,
+		GladeProject *project)
+{
+	if (glade_project_verify (project, FALSE))
+	{
+		gchar *name = glade_project_get_name (project);
+		glade_util_ui_message (glade_app_get_window(),
+				       GLADE_UI_INFO, NULL,
+				       _("Project %s has no deprecated widgets "
+					 "or version mismatches."),
+				       name);
+		g_free (name);
+	}
+}
+
+static GtkWidget *
+glade_project_build_prefs_box (GladeProject *project)
+{
+	GtkWidget *main_box, *button;
+	GtkWidget *vbox, *hbox, *frame;
+	GtkWidget *glade_radio, *builder_radio, *target_radio, *active_radio;
+	GtkWidget *label, *alignment;
+	GList     *list, *targets;
+	gchar     *string = g_strdup_printf ("<b>%s</b>", _("File format"));
+
+	main_box = gtk_vbox_new (FALSE, 0);
+
+	/* Target versions */
+	string = g_strdup_printf ("<b>%s</b>", _("Target Versions:"));
+	frame = gtk_frame_new (NULL);
+	vbox = gtk_vbox_new (FALSE, 0);
+	alignment = gtk_alignment_new (0.5F, 0.5F, 1.0F, 1.0F);
+	
+	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 2, 0, 12, 0);
+	
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
+
+	label = gtk_label_new (string);
+	g_free (string);
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	
+	gtk_frame_set_label_widget (GTK_FRAME (frame), label);
+	gtk_container_add (GTK_CONTAINER (alignment), vbox);
+	gtk_container_add (GTK_CONTAINER (frame), alignment);
+	
+	gtk_box_pack_start (GTK_BOX (main_box), frame, TRUE, TRUE, 2);
+
+	/* Add stuff to vbox */
+	for (list = glade_app_get_catalogs (); list; list = list->next)
+	{
+		GladeCatalog *catalog = list->data;
+		gint          minor, major;
+
+		/* Skip if theres only one option */
+		if (g_list_length (glade_catalog_get_targets (catalog)) <= 1)
+			continue;
+
+		glade_project_get_target_version (project,
+						  glade_catalog_get_name (catalog),
+						  &major,
+						  &minor);
+
+		string = g_strdup_printf (_("%s catalog"), 
+					  glade_catalog_get_name (catalog));
+		label = gtk_label_new (string);
+		g_free (string);
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0F, 0.5F);
+		
+		gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 2);
+		hbox = gtk_hbox_new (FALSE, 0);
+
+		active_radio = NULL;
+		target_radio = NULL;
+		
+		for (targets = glade_catalog_get_targets (catalog); 
+		     targets; targets = targets->next)
+		{
+			GladeTargetableVersion *version = targets->data;
+			gchar     *name = g_strdup_printf ("%d.%d", 
+							   version->major,
+							   version->minor);
+
+			if (!target_radio)
+				target_radio = gtk_radio_button_new_with_label (NULL, name);
+			else
+				target_radio = 
+					gtk_radio_button_new_with_label_from_widget 
+					(GTK_RADIO_BUTTON (target_radio), name);
+			g_free (name);
+
+			g_signal_connect (G_OBJECT (target_radio), "clicked",
+					  G_CALLBACK (target_button_clicked), project);
+
+			g_object_set_data (G_OBJECT (target_radio), "version", version);
+			g_object_set_data (G_OBJECT (target_radio), "catalog", 
+					   (gchar *)glade_catalog_get_name (catalog));
+
+			gtk_box_pack_end (GTK_BOX (hbox), target_radio, TRUE, TRUE, 2);
+
+			if (major == version->major &&
+			    minor == version->minor)
+				active_radio = target_radio;
+			
+		}
+
+		if (active_radio)
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (active_radio), TRUE);
+		else 
+			g_warning ("Corrupt catalog versions");
+
+		gtk_box_pack_end (GTK_BOX (vbox), hbox, TRUE, TRUE, 2);
+
+	}
+
+	/* Project format */
+	string = g_strdup_printf ("<b>%s</b>", _("File format"));
+	frame = gtk_frame_new (NULL);
+	hbox = gtk_hbox_new (FALSE, 0);
+	alignment = gtk_alignment_new (0.5F, 0.5F, 0.8F, 0.8F);
+
+	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 2, 0, 12, 0);
+
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
+
+	label = gtk_label_new (string);
+	g_free (string);
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+
+	glade_radio = gtk_radio_button_new_with_label (NULL, "Libglade");
+	builder_radio = gtk_radio_button_new_with_label_from_widget
+		(GTK_RADIO_BUTTON (glade_radio), "GtkBuilder");
+
+	if (glade_project_get_format (project) == GLADE_PROJECT_FORMAT_GTKBUILDER)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (builder_radio), TRUE);
+	else
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (glade_radio), TRUE);
+
+	g_signal_connect (G_OBJECT (glade_radio), "clicked",
+			  G_CALLBACK (format_libglade_button_clicked), project);
+
+	g_signal_connect (G_OBJECT (builder_radio), "clicked",
+			  G_CALLBACK (format_builder_button_clicked), project);
+
+	gtk_box_pack_start (GTK_BOX (hbox), builder_radio, TRUE, TRUE, 2);
+	gtk_box_pack_start (GTK_BOX (hbox), glade_radio, TRUE, TRUE, 2);
+
+	gtk_frame_set_label_widget (GTK_FRAME (frame), label);
+	gtk_container_add (GTK_CONTAINER (alignment), hbox);
+	gtk_container_add (GTK_CONTAINER (frame), alignment);
+
+	gtk_box_pack_start (GTK_BOX (main_box), frame, TRUE, TRUE, 2);
+
+
+	/* Run verify */
+	string = g_strdup_printf ("<b>%s</b>", _("Verify versions and deprications:"));
+	frame = gtk_frame_new (NULL);
+	alignment = gtk_alignment_new (0.5F, 0.5F, 1.0F, 1.0F);
+	
+	button = gtk_button_new_from_stock (GTK_STOCK_EXECUTE);
+	g_signal_connect (G_OBJECT (button), "clicked", 
+			  G_CALLBACK (verify_clicked), project);
+	
+	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 2, 0, 12, 0);
+	
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
+
+	label = gtk_label_new (string);
+	g_free (string);
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	
+	gtk_frame_set_label_widget (GTK_FRAME (frame), label);
+	gtk_container_add (GTK_CONTAINER (alignment), button);
+	gtk_container_add (GTK_CONTAINER (frame), alignment);
+	
+	gtk_box_pack_start (GTK_BOX (main_box), frame, FALSE, FALSE, 4);
+
+
+	gtk_widget_show_all (main_box);
+
+	return main_box;
+}
+
+/**
+ * glade_project_preferences:
+ * @project: A #GladeProject
+ *
+ * Runs a preferences dialog for @project.
+ */
+void
+glade_project_preferences (GladeProject *project)
+{
+
+	GtkWidget      *dialog, *widget;
+	gchar          *title, *name;
+
+
+	g_return_if_fail (GLADE_IS_PROJECT (project));
+
+	name = glade_project_get_name (project);
+	title = g_strdup_printf ("%s preferences", name);
+
+	dialog = gtk_dialog_new_with_buttons (title,
+					      GTK_WINDOW (glade_app_get_window ()),
+					      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					      GTK_STOCK_OK,
+					      GTK_RESPONSE_ACCEPT,
+					      NULL);
+	g_free (title);
+	g_free (name);
+
+	widget = glade_project_build_prefs_box (project);
+	gtk_box_pack_end (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
+			  widget, TRUE, TRUE, 2);
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+}
